@@ -17,16 +17,19 @@
  * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
  * Boston, MA 02111-1307, USA.
  */
- 
+
 #ifndef GLOM_DATASTRUCTURE_LAYOUTITEM_FIELD_H
 #define GLOM_DATASTRUCTURE_LAYOUTITEM_FIELD_H
 
 #include "layoutitem.h"
+#include "usesrelationship.h"
 #include "../field.h"
 #include "../numeric_format.h"
 #include "../relationship.h"
 
-class LayoutItem_Field : public LayoutItem
+class LayoutItem_Field 
+ : public LayoutItem,
+   public UsesRelationship
 {
 public:
 
@@ -51,14 +54,14 @@ public:
    */
   virtual Glib::ustring get_layout_display_name() const;
 
-  virtual bool get_has_relationship_name() const;
-  virtual Glib::ustring get_relationship_name() const;
-  //virtual void set_relationship_name(const Glib::ustring& relationship_name);
-
   virtual Glib::ustring get_part_type_name() const;
 
-  //This is filled in by looking at the database structure:
-  Field m_field;
+  void set_full_field_details(const sharedptr<const Field>& field);
+  sharedptr<const Field> get_full_field_details() const;
+
+  ///Convenience function, to avoid use of get_full_field_details().
+  Field::glom_field_type get_glom_type() const;
+
   //TODO: This might occasionally be different on different layouts: Glib::ustring m_title;
 
 
@@ -72,10 +75,6 @@ public:
   bool m_priv_view;
   bool m_priv_edit;
 
-  //TODO_Performance: This is just cached data, so we don't need to always lookup the relationship details from the document, from the name.
-  //Maybe use a smartpointer?
-  Relationship m_relationship;
-
   FieldFormatting m_formatting;
 
   bool get_formatting_use_default() const;
@@ -85,7 +84,10 @@ public:
 
 protected:
 
-  //Glib::ustring m_relationship_name; //bool m_related;
+  //This is just a cache, filled in by looking at the database structure:
+  sharedptr<const Field> m_field;
+  bool m_field_cache_valid; //Whetehr m_field is up-to-date.
+
   bool m_hidden;
   bool m_formatting_use_default;
 };
