@@ -23,7 +23,7 @@
 #include "dialog_choose_user.h"
 #include <bakery/App/App_Gtk.h> //For util_bold_message().
 //#include <libgnome/gnome-i18n.h>
-#include <libintl.h>
+#include <glibmm/i18n.h>
 
 Dialog_UsersList::Dialog_UsersList(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade)
 : Gtk::Dialog(cobject),
@@ -60,7 +60,7 @@ Dialog_UsersList::Dialog_UsersList(BaseObjectType* cobject, const Glib::RefPtr<G
 
 
     // Append the View columns:
-    m_treeview_users->append_column(gettext("User"), m_model_columns_users.m_col_name);
+    m_treeview_users->append_column(_("User"), m_model_columns_users.m_col_name);
   }
 
 
@@ -136,7 +136,7 @@ void Dialog_UsersList::set_document(const Glib::ustring& layout, Document_Glom*
     //guint group_sequence = 1; //0 means no sequence
     for(Document_Glom::type_mapLayoutGroupSequence::const_iterator iter = mapGroups.begin(); iter != mapGroups.end(); ++iter)
     {
-      const LayoutGroup& group = iter->second;
+      sharedptr<const LayoutGroup> group = iter->second;
 
       add_group(Gtk::TreeModel::iterator(), group);
     }
@@ -217,8 +217,8 @@ void Dialog_UsersList::on_button_user_delete()
         const Glib::ustring user = row[m_model_columns_users.m_col_name];
         if(!user.empty())
         {
-          Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(gettext("Delete User")), true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_OK_CANCEL);
-          dialog.set_secondary_text(gettext("Are your sure that you wish to delete this user?"));
+          Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Delete User")), true, Gtk::MESSAGE_QUESTION, Gtk::BUTTONS_OK_CANCEL);
+          dialog.set_secondary_text(_("Are your sure that you wish to delete this user?"));
           dialog.set_transient_for(*this);
 
           int response = dialog.run();
@@ -278,7 +278,7 @@ void Dialog_UsersList::on_button_user_add()
 
     for(Document_Glom::type_listTableInfo::const_iterator iter = table_list.begin(); iter != table_list.end(); ++iter)
     {
-      Glib::ustring strQuery = "REVOKE ALL PRIVILEGES ON " + iter->get_name() + " FROM " + user;
+      Glib::ustring strQuery = "REVOKE ALL PRIVILEGES ON \"" + (*iter)->get_name() + "\" FROM " + user;
       Query_execute(strQuery);
     }
 
@@ -454,8 +454,8 @@ bool Dialog_UsersList::warn_about_empty_standard_group()
   {
     if(m_model_users->children().size() == 1)
     {
-      Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(gettext("Developer group may not be empty.")), true, Gtk::MESSAGE_WARNING);
-      dialog.set_secondary_text(gettext("The developer group must contain at least one user."));
+      Gtk::MessageDialog dialog(Bakery::App_Gtk::util_bold_message(_("Developer group may not be empty.")), true, Gtk::MESSAGE_WARNING);
+      dialog.set_secondary_text(_("The developer group must contain at least one user."));
       dialog.set_transient_for(*this);
       dialog.run();
 
