@@ -24,6 +24,7 @@
 #include "gtkmm.h"
 #include "../../data_structure/field.h"
 #include <libgdamm.h>
+#include <libglademm.h>
 
 #include <vector>
 #include <map>
@@ -65,6 +66,7 @@ public:
   friend class InnerIgnore; //declared below.
 
   AddDel();
+  AddDel(BaseObjectType* cobject, const Glib::RefPtr<Gnome::Glade::Xml>& refGlade);
   virtual ~AddDel();
 
   virtual void set_allow_user_actions(bool bVal = true);
@@ -72,21 +74,21 @@ public:
 
   virtual void set_allow_add(bool val = true);
   virtual void set_allow_delete(bool val = true);
-    
+
   virtual void set_allow_column_chooser(bool value = true);
-  
+
   virtual Gtk::TreeModel::iterator add_item(const Glib::ustring& strKey); //Return index of new row.
 
   /** Get an iterator to the blank row in which the user should add data for the new row.
    * You can then add the row to your underlying data store when some data has been filled, by handling signal_user_changed.
    */
   virtual Gtk::TreeModel::iterator get_item_placeholder(); //Return index of the placeholder row.
-  
+
   virtual void remove_item(const Gtk::TreeModel::iterator& iter);
+  virtual void remove_item_by_key(const Glib::ustring& strKey);
 
   virtual void remove_all();
 
-  virtual Gnome::Gda::Value get_value_as_value(const Gtk::TreeModel::iterator& iter, guint col );
   virtual Glib::ustring get_value(const Gtk::TreeModel::iterator& iter, guint col);
   virtual bool get_value_as_bool(const Gtk::TreeModel::iterator& iter, guint col);
 
@@ -94,35 +96,29 @@ public:
    */
   virtual Glib::ustring get_value_key(const Gtk::TreeModel::iterator& iter);
 
-  /** Get the row's hidden key
-   */
-  virtual Gnome::Gda::Value get_value_key_as_value(const Gtk::TreeModel::iterator& iter);
-
   /** Set the row's hidden key
    */
   virtual void set_value_key(const Gtk::TreeModel::iterator& iter, const Glib::ustring& strValue);
 
   virtual Glib::ustring get_value_selected(guint col);
-  virtual Gnome::Gda::Value get_value_selected_as_value(guint col);
   virtual Glib::ustring get_value_key_selected();
-  virtual Gnome::Gda::Value get_value_key_selected_as_value();
-  
+
   virtual Gtk::TreeModel::iterator get_item_selected();
 
   virtual bool select_item(const Gtk::TreeModel::iterator& iter, guint column, bool start_editing = false);  //bool indicates success.
   virtual bool select_item(const Gtk::TreeModel::iterator& iter);
-  
+
   //Select row with this key value:
   virtual bool select_item(const Glib::ustring& strItemText, guint column, bool start_editing = false);
-  
+
   virtual guint get_count() const;
 
-  virtual void set_value(const Gtk::TreeModel::iterator& iter, guint col, const Gnome::Gda::Value& value);
+  //virtual void set_value(const Gtk::TreeModel::iterator& iter, guint col, const Gnome::Gda::Value& value);
   virtual void set_value(const Gtk::TreeModel::iterator& iter, guint col, const Glib::ustring& strValue);
   virtual void set_value(const Gtk::TreeModel::iterator& iter, guint col, unsigned long ulValue);
   virtual void set_value(const Gtk::TreeModel::iterator& iter, guint col, bool bVal);
 
-  virtual void set_value_selected(guint col, const Gnome::Gda::Value& value);
+  //virtual void set_value_selected(guint col, const Gnome::Gda::Value& value);
 
   virtual bool get_is_first_row(const Gtk::TreeModel::iterator& iter) const;
   virtual bool get_is_last_row(const Gtk::TreeModel::iterator& iter) const;
@@ -130,29 +126,24 @@ public:
   /** @result Whether this is a blank row where date for a new row should be entered
    */
   virtual bool get_is_placeholder_row(const Gtk::TreeModel::iterator& iter) const;
-  
-  virtual void set_select_text(const Glib::ustring& strVal);
-  virtual Glib::ustring get_select_text() const;
 
-  //Use this in order to use get_value_key_as_value().
-  virtual void set_key_type(const Field& field);
- 
+
   virtual guint add_column(const AddDelColumnInfo& column_info);
   virtual guint add_column(const Glib::ustring& strTitle, AddDelColumnInfo::enumStyles style = AddDelColumnInfo::STYLE_Text, bool editable = true, bool visible = true);
   virtual guint add_column(const Glib::ustring& strTitle, const Glib::ustring& column_id, AddDelColumnInfo::enumStyles style = AddDelColumnInfo::STYLE_Text, bool editable = true, bool visible = true);
-  virtual guint add_column(const Field& field, AddDelColumnInfo::enumStyles style = AddDelColumnInfo::STYLE_Text, bool editable = true, bool visible = true);
+
 
   virtual guint get_columns_count() const;
 
   virtual Glib::ustring get_column_field(guint column_index) const;
-  
+
   typedef AddDelColumnInfo::type_vecStrings type_vecStrings;
 
   /** Retrieves the column order, even after they have been reordered by the user.
    * @result a vector of column_id. These column_ids were provided in the call to add_column().
    */
   virtual type_vecStrings get_columns_order() const;
-  
+
   virtual void remove_all_columns();
   //virtual void set_columns_count(guint count);
   //virtual void set_column_title(guint col, const Glib::ustring& strText);
@@ -160,7 +151,7 @@ public:
 
   /// For popup cells.
   virtual void set_column_choices(guint col, const type_vecStrings& vecStrings);
-   
+
   virtual void construct_specified_columns(); //Delay actual use of set_column_*() stuff until this method is called.
 
   virtual void set_show_column_titles(bool bVal = true);
@@ -181,7 +172,7 @@ public:
   Glib::RefPtr<const Gtk::TreeModel> get_model() const;
 
   virtual void set_rules_hint(bool val = true);
-      
+
   //Signals:
   //row number.
   typedef sigc::signal<void, const Gtk::TreeModel::iterator&> type_signal_user_added;
@@ -213,14 +204,15 @@ public:
 
   virtual Gtk::TreeModel::iterator get_last_row();
   virtual Gtk::TreeModel::iterator get_last_row() const;
-  
+
 protected:
+  void init();
 
   /** Get an iterator to the blank row in which the user should add data for the new row.
    * You can then add the row to your underlying data store when some data has been filled, by handling signal_user_changed.
    */
   virtual Gtk::TreeModel::iterator add_item_placeholder(); //Return index of new row.
-  
+ 
   virtual void setup_menu();
   virtual Glib::ustring treeview_get_key(const Gtk::TreeModel::iterator& row);
 
@@ -228,14 +220,14 @@ protected:
   virtual Gtk::TreeModel::iterator get_next_available_row_with_add_if_necessary();
   virtual void add_blank();
 
-  
+
   //Signal handlers:
   virtual void on_treeview_cell_edited(const Glib::ustring& path_string, const Glib::ustring& new_text, int model_column_index);
   virtual void on_treeview_cell_edited_bool(const Glib::ustring& path_string, int model_column_index);
-  
+
   virtual bool on_treeview_column_drop(Gtk::TreeView* treeview, Gtk::TreeViewColumn* column, Gtk::TreeViewColumn* prev_column, Gtk::TreeViewColumn* next_column);
   virtual void on_treeview_columns_changed();
-  
+
   virtual bool on_button_press_event_Popup(GdkEventButton* event);
 
   virtual void on_MenuPopup_activate_Edit();
@@ -268,9 +260,6 @@ protected:
   template<class T_ModelColumnType>
   guint treeview_append_column(const Glib::ustring& title, const Gtk::TreeModelColumn<T_ModelColumnType>& column, const Glib::ustring& column_id);
 
-
-  static Glib::ustring string_escape_underscores(const Glib::ustring& text);
-  
   typedef Gtk::VBox type_base;
 
   //Member widgets:
@@ -287,8 +276,6 @@ protected:
 
   Gtk::Menu m_MenuPopup;
 
-  Glib::ustring m_strSelectText; //e.g. 'Edit', 'Use'.
-
   bool m_bAllowUserActions;
 
   bool m_bPreventUserSignals;
@@ -302,8 +289,6 @@ protected:
   bool m_auto_add;
   bool m_allow_add;
   bool m_allow_delete;
-
-  Field m_key_field;
   
   //signals:
   type_signal_user_added m_signal_user_added;
@@ -327,13 +312,6 @@ protected:
     bool m_bPreventUserSignals, m_bIgnoreSheetSignals;
   };
 
-/*
-  class DynamicColumnRecord : public Gtk::TreeModel::ColumnRecord
-  {
-    typedef std::vector<Gtk::TreeModelColumnBase> type_vecColumns;
-    type_vecColumns m_vecColumns;
-  };
-*/
 };
 
 template<class T_ModelColumnType>
