@@ -23,10 +23,13 @@
 #include "comboentryglom.h"
 #include "comboglom.h"
 #include "textviewglom.h"
+#include "imageglom.h"
 #include "../data_structure/glomconversions.h"
 #include "../application.h"
 #include "../mode_data/dialog_choose_field.h"
-#include "../mode_data/dialog_field_layout.h"
+#include "dialog_choose_id.h"
+#include "../layout_item_dialogs/dialog_field_layout.h"
+#include "../utils.h"
 #include <glibmm/i18n.h>
 
 /*
@@ -35,13 +38,14 @@ DataWidget::DataWidget(Field::glom_field_type glom_type, const Glib::ustring& ti
   m_pMenuPopup(0)
 */
 
-DataWidget::DataWidget(const LayoutItem_Field& field, const Glib::ustring& table_name, const Document_Glom* document)
+DataWidget::DataWidget(const sharedptr<LayoutItem_Field>& field, const Glib::ustring& table_name, const Document_Glom* document)
 {
-  Field::glom_field_type glom_type = field.m_field.get_glom_type();
-  set_layout_item(field.clone(), table_name); //takes ownership
+  const Field::glom_field_type glom_type = field->get_glom_type();
+  set_layout_item(field, table_name);
 
-  Gtk::Widget* child = 0;
-  const Glib::ustring title = field.m_field.get_title_or_name();
+  m_child = 0;
+  LayoutWidgetField* pFieldWidget = 0;
+  const Glib::ustring title = field->get_title_or_name();
   if(glom_type == Field::TYPE_BOOLEAN)
   {
     Gtk::CheckButton* checkbutton = Gtk::manage( new Gtk::CheckButton( title ) );
@@ -50,71 +54,86 @@ DataWidget::DataWidget(const LayoutItem_Field& field, const Glib::ustring& table
 
     //TODO: entry->signal_user_requested_layout().connect( sigc::mem_fun(*this, &DataWidget::on_child_user_requested_layout );
 
-    child = checkbutton;
+    m_child = checkbutton;
 
     m_label.set_text( Glib::ustring() ); //It is not used.
   }
+  else if(glom_type == Field::TYPE_IMAGE)
+  {
+    ImageGlom* image = Gtk::manage( new ImageGlom() );
+    image->set_size_request(200, 200);
+    //Gtk::Image* image = Gtk::manage( new Gtk::Image("/home/murrayc/gnome-small.jpg") );
+    image->show();
+    //TODO: Respond to double-click: checkbutton->signal_toggled().connect( sigc::mem_fun(*this, &DataWidget::on_widget_edited)  );
+
+    //TODO: entry->signal_user_requested_layout().connect( sigc::mem_fun(*this, &DataWidget::on_child_user_requested_layout );
+
+    m_child = image;
+    pFieldWidget = image;
+
+    m_label.set_label(title);
+    m_label.set_alignment(0);
+    m_label.show();
+  }
   else
   {
     m_label.set_label(title);
     m_label.set_alignment(0);
     m_label.show();
 
-    LayoutWidgetField* pFieldWidget = 0;
     //Use a Combo if there is a drop-down of choices (A "value list"), else an Entry:
-    if(field.get_has_choices())
+    if(field->get_formatting_used().get_has_choices())
     {
       ComboGlomChoicesBase* combo = 0; //Gtk::manage(new ComboEntryGlom());
 
-      if(field.get_has_custom_choices())
+      if(field->get_formatting_used().get_has_custom_choices())
       {
-        if(field.get_choices_restricted())
+        if(field->get_formatting_used().get_choices_restricted())
           combo = Gtk::manage(new ComboGlom());
         else
           combo = Gtk::manage(new ComboEntryGlom());
 
         //set_choices() needs this, for the numeric layout:
-        combo->set_layout_item(get_layout_item()->clone(), table_name); //TODO_Performance: We only need this for the numerical format.
+        combo->set_layout_item( get_layout_item(), table_name);
 
-        combo->set_choices( field.get_choices_custom() );
+        combo->set_choices( field->get_formatting_used().get_choices_custom() );
       }
-      else if(field.get_has_related_choices())
+      else if(field->get_formatting_used().get_has_related_choices())
       {
-        Glib::ustring choice_relationship_name, choice_field, choice_second;
-        field.get_choices(choice_relationship_name, choice_field, choice_second);
-        if(!choice_relationship_name.empty() && !choice_field.empty())
+        sharedptr<Relationship> choice_relationship;
+        Glib::ustring choice_field, choice_second;
+        field->get_formatting_used().get_choices(choice_relationship, choice_field, choice_second);
+        if(choice_relationship && !choice_field.empty())
         {
-          const Relationship relationship = field.m_choices_related_relationship;
-          const Glib::ustring to_table = relationship.get_to_table();
+          const Glib::ustring to_table = choice_relationship->get_to_table();
 
           const bool with_second = !choice_second.empty();
 
           if(with_second && document)
           {
-            Field field_second;
-            document->get_field(to_table, choice_second, field_second);
+            sharedptr<Field> field_second = document->get_field(to_table, choice_second);
 
-            LayoutItem_Field layout_field_second;
-            layout_field_second.m_field = field_second;
-            //We use the default formatting for this field.
+            sharedptr<LayoutItem_Field> layout_field_second = sharedptr<LayoutItem_Field>::create();
+            layout_field_second->set_full_field_details(field_second);
+            //We use the default formatting for this field->
 
-            if(field.get_choices_restricted())
+            if(field->get_formatting_used().get_choices_restricted())
               combo = Gtk::manage(new ComboGlom(layout_field_second));
             else
               combo = Gtk::manage(new ComboEntryGlom(layout_field_second));
           }
           else
           {
-            if(field.get_choices_restricted())
+            if(field->get_formatting_used().get_choices_restricted())
               combo = Gtk::manage(new ComboGlom());
             else
               combo = Gtk::manage(new ComboEntryGlom());
           }
 
           //set_choices() needs this, for the numeric layout:
-          combo->set_layout_item(get_layout_item()->clone(), table_name);
+          combo->set_layout_item( get_layout_item(), table_name);
 
-          combo->set_choices_with_second( get_choice_values(field) );
+          combo->set_choices_with_second( GlomUtils::get_choice_values(field) );
         }
       }
       else
@@ -126,7 +145,7 @@ DataWidget::DataWidget(const LayoutItem_Field& field, const Glib::ustring& table
     }
     else
     {
-      if(field.get_text_format_multiline())
+      if(field->get_formatting_used().get_text_format_multiline())
       {
         TextViewGlom* textview = Gtk::manage(new TextViewGlom(glom_type));
         pFieldWidget = textview;
@@ -137,9 +156,12 @@ DataWidget::DataWidget(const LayoutItem_Field& field, const Glib::ustring& table
         pFieldWidget = entry;
       }
 
-      pFieldWidget->set_layout_item(get_layout_item()->clone(), table_name); //TODO_Performance: We only need this for the numerical format.
+      pFieldWidget->set_layout_item( get_layout_item(), table_name); //TODO_Performance: We only need this for the numerical format.
     }
-
+  }
+  
+  if(pFieldWidget)
+  {
     pFieldWidget->signal_edited().connect( sigc::mem_fun(*this, &DataWidget::on_widget_edited)  );
 
     pFieldWidget->signal_user_requested_layout().connect( sigc::mem_fun(*this, &DataWidget::on_child_user_requested_layout) );
@@ -147,14 +169,45 @@ DataWidget::DataWidget(const LayoutItem_Field& field, const Glib::ustring& table
     pFieldWidget->signal_layout_item_added().connect( sigc::mem_fun(*this, &DataWidget::on_child_layout_item_added) );
 
 
-    child = dynamic_cast<Gtk::Widget*>(pFieldWidget);
-    int width = get_suitable_width(field);
-    child->set_size_request(width, -1 /* auto */);
-    child->show_all();
+    m_child = dynamic_cast<Gtk::Widget*>(pFieldWidget);
+    const int width = get_suitable_width(field);
+
+    if(glom_type == Field::TYPE_IMAGE) //GtkImage widgets default to no size (invisible) if they are empty.
+      m_child->set_size_request(width, width);
+    else
+      m_child->set_size_request(width, -1 /* auto */);
+
+    m_child->show_all();
   }
 
-  if(child)
-    add(*child);
+  if(m_child)
+  {
+    bool child_added = false; //Don't use an extra container unless necessary.
+    if(document->get_field_used_in_relationship_to_one(table_name, field->get_name()))
+    {
+      Gtk::HBox* hbox_parent = Gtk::manage( new Gtk::HBox() ); //We put the child (and any extra stuff) in this:
+      hbox_parent->set_spacing(6);
+
+      hbox_parent->pack_start(*m_child);
+      add(*hbox_parent);
+
+
+      Gtk::Button* button_go_to_details = Gtk::manage(new Gtk::Button(Gtk::Stock::OPEN));
+      hbox_parent->pack_start(*button_go_to_details);
+      button_go_to_details->signal_clicked().connect(sigc::mem_fun(*this, &DataWidget::on_button_open_details));
+
+      Gtk::Button* button_select = Gtk::manage(new Gtk::Button(Gtk::Stock::FIND));
+      hbox_parent->pack_start(*button_select);
+      button_select->signal_clicked().connect(sigc::mem_fun(*this, &DataWidget::on_button_select_id));
+
+      child_added = true;
+    }
+
+    if(!child_added)
+      add(*m_child);
+  }
+
+
 
   setup_menu();
 
@@ -178,7 +231,7 @@ DataWidget::type_signal_edited DataWidget::signal_edited()
 
 void DataWidget::set_value(const Gnome::Gda::Value& value)
 {
-  Gtk::Widget* widget = get_child();
+  Gtk::Widget* widget = get_data_child_widget();
   LayoutWidgetField* generic_field_widget = dynamic_cast<LayoutWidgetField*>(widget);
   if(generic_field_widget)
     generic_field_widget->set_value(value);
@@ -198,7 +251,7 @@ void DataWidget::set_value(const Gnome::Gda::Value& value)
 
 Gnome::Gda::Value DataWidget::get_value() const
 {
-  const Gtk::Widget* widget = get_child();
+  const Gtk::Widget* widget = get_data_child_widget();
   const LayoutWidgetField* generic_field_widget = dynamic_cast<const LayoutWidgetField*>(widget);
   if(generic_field_widget)
     return generic_field_widget->get_value();
@@ -224,11 +277,11 @@ const Gtk::Label* DataWidget::get_label() const
   return &m_label;
 }
 
-int DataWidget::get_suitable_width(const LayoutItem_Field& field_layout)
+int DataWidget::get_suitable_width(const sharedptr<const LayoutItem_Field>& field_layout)
 {
   int result = 150;
 
-  const Field::glom_field_type field_type = field_layout.m_field.get_glom_type();
+  const Field::glom_field_type field_type = field_layout->get_glom_type();
 
   Glib::ustring example_text;
   switch(field_type)
@@ -253,12 +306,13 @@ int DataWidget::get_suitable_width(const LayoutItem_Field& field_layout)
     }
     case(Field::TYPE_NUMERIC):
     {
-      example_text = "9999999999";
+      example_text = "EUR 9999999999";
       break;
     }
     case(Field::TYPE_TEXT):
+    case(Field::TYPE_IMAGE): //Give images the same width as text fields, so they will often line up.
     {
-      //if(!field_layout.get_text_format_multiline()) //Use the full width for multi-line text.
+      //if(!field_layout->get_text_format_multiline()) //Use the full width for multi-line text.
         example_text = "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAA";
       break;
     }
@@ -287,7 +341,7 @@ int DataWidget::get_suitable_width(const LayoutItem_Field& field_layout)
 
 void DataWidget::set_viewable(bool viewable)
 {
-  Gtk::Widget* child = get_child();
+  Gtk::Widget* child = get_data_child_widget();
   Gtk::Entry* entry = dynamic_cast<Gtk::Entry*>(child);
   if(entry)
     entry->set_visibility(viewable); //TODO: This is not an ideal way to show non-viewable fields..
@@ -301,7 +355,7 @@ void DataWidget::set_viewable(bool viewable)
 
 void DataWidget::set_editable(bool editable)
 {
-  Gtk::Widget* child = get_child();
+  Gtk::Widget* child = get_data_child_widget();
   Gtk::Entry* entry = dynamic_cast<Gtk::Entry*>(child);
   if(entry)
     entry->set_editable(editable);
@@ -323,13 +377,13 @@ void DataWidget::setup_menu()
     sigc::mem_fun(*this, &DataWidget::on_menupopup_activate_layout_properties) );
 
   m_refActionGroup->add(m_refContextAddField,
-    sigc::bind( sigc::mem_fun(*this, &DataWidget::on_menupopup_add_item), TreeStore_Layout::TYPE_FIELD ) );
+    sigc::bind( sigc::mem_fun(*this, &DataWidget::on_menupopup_add_item), LayoutWidgetBase::TYPE_FIELD ) );
 
   m_refActionGroup->add(m_refContextAddRelatedRecords,
-    sigc::bind( sigc::mem_fun(*this, &DataWidget::on_menupopup_add_item), TreeStore_Layout::TYPE_PORTAL ) );
+    sigc::bind( sigc::mem_fun(*this, &DataWidget::on_menupopup_add_item), LayoutWidgetBase::TYPE_PORTAL ) );
 
   m_refActionGroup->add(m_refContextAddGroup,
-    sigc::bind( sigc::mem_fun(*this, &DataWidget::on_menupopup_add_item), TreeStore_Layout::TYPE_GROUP ) );
+    sigc::bind( sigc::mem_fun(*this, &DataWidget::on_menupopup_add_item), LayoutWidgetBase::TYPE_GROUP ) );
 
   //TODO: This does not work until this widget is in a container in the window:s
   App_Glom* pApp = get_application();
@@ -389,7 +443,7 @@ void DataWidget::setup_menu()
 
 bool DataWidget::on_button_press_event(GdkEventButton *event)
 {
-  g_warning("DataWidget::on_button_press_event_popup");
+  //g_warning("DataWidget::on_button_press_event_popup");
 
   //Enable/Disable items.
   //We did this earlier, but get_application is more likely to work now:
@@ -421,9 +475,14 @@ bool DataWidget::on_button_press_event(GdkEventButton *event)
   return Gtk::EventBox::on_button_press_event(event);
 }
 
-bool DataWidget::offer_field_list(const Glib::ustring& table_name, LayoutItem_Field& field)
+sharedptr<LayoutItem_Field> DataWidget::offer_field_list(const Glib::ustring& table_name)
 {
-  bool result = false;
+  return offer_field_list(table_name, sharedptr<LayoutItem_Field>());
+}
+
+sharedptr<LayoutItem_Field> DataWidget::offer_field_list(const Glib::ustring& table_name, const sharedptr<const LayoutItem_Field>& start_field)
+{
+  sharedptr<LayoutItem_Field> result;
 
   try
   {
@@ -434,14 +493,14 @@ bool DataWidget::offer_field_list(const Glib::ustring& table_name, LayoutItem_Fi
 
     if(dialog)
     {
-      dialog->set_document(get_document(), table_name, field);
+      dialog->set_document(get_document(), table_name, start_field);
       dialog->set_transient_for(*get_application());
       int response = dialog->run();
       dialog->hide();
       if(response == Gtk::RESPONSE_OK)
       {
         //Get the chosen field:
-        result = dialog->get_field_chosen(field);
+        result = dialog->get_field_chosen();
       }
 
       delete dialog;
@@ -455,9 +514,9 @@ bool DataWidget::offer_field_list(const Glib::ustring& table_name, LayoutItem_Fi
   return result;
 }
 
-bool DataWidget::offer_field_layout( LayoutItem_Field& field)
+sharedptr<LayoutItem_Field> DataWidget::offer_field_layout(const sharedptr<const LayoutItem_Field>& start_field)
 {
-  bool result = false;
+  sharedptr<LayoutItem_Field> result;
 
   try
   {
@@ -469,14 +528,14 @@ bool DataWidget::offer_field_layout( LayoutItem_Field& field)
     if(dialog)
     {
       add_view(dialog); //Give it access to the document.
-      dialog->set_field(field, m_table_name);
+      dialog->set_field(start_field, m_table_name);
       dialog->set_transient_for(*get_application());
-      int response = dialog->run();
+      const int response = dialog->run();
       dialog->hide();
       if(response == Gtk::RESPONSE_OK)
       {
         //Get the chosen field:
-        result = dialog->get_field_chosen(field);
+        result = dialog->get_field_chosen();
       }
 
       remove_view(dialog);
@@ -492,7 +551,7 @@ bool DataWidget::offer_field_layout( LayoutItem_Field& field)
 }
 
 /*
-void DataWidget::on_menupopup_add_item(TreeStore_Layout::enumType item)
+void DataWidget::on_menupopup_add_item(LayoutWidgetBase::enumType item)
 {
   signal_layout_item_added().emit(item);
 }
@@ -502,12 +561,13 @@ void DataWidget::on_menupopup_activate_layout()
 {
   //finish_editing();
 
-  LayoutItem_Field* layoutField = dynamic_cast<LayoutItem_Field*>(get_layout_item());
+  sharedptr<LayoutItem_Field> layoutField = sharedptr<LayoutItem_Field>::cast_dynamic(get_layout_item());
   if(layoutField)
   {
-    bool test = offer_field_list(m_table_name, *layoutField);
-    if(test)
+    sharedptr<LayoutItem_Field> itemchosen = offer_field_list(m_table_name, layoutField);
+    if(itemchosen)
     {
+      *layoutField = *itemchosen;
       signal_layout_changed().emit();
     }
   }
@@ -517,12 +577,13 @@ void DataWidget::on_menupopup_activate_layout_properties()
 {
   //finish_editing();
 
-  LayoutItem_Field* layoutField = dynamic_cast<LayoutItem_Field*>(get_layout_item());
+  sharedptr<LayoutItem_Field> layoutField = sharedptr<LayoutItem_Field>::cast_dynamic(get_layout_item());
   if(layoutField)
   {
-    bool test = offer_field_layout(*layoutField);
-    if(test)
+    sharedptr<LayoutItem_Field> itemchosen = offer_field_layout(layoutField);
+    if(itemchosen)
     {
+      set_layout_item(itemchosen, m_table_name);
       signal_layout_changed().emit();
     }
   }
@@ -546,8 +607,93 @@ App_Glom* DataWidget::get_application()
   return dynamic_cast<App_Glom*>(pWindow);
 }
 
-void DataWidget::on_child_layout_item_added(TreeStore_Layout::enumType item_type)
+void DataWidget::on_child_layout_item_added(LayoutWidgetBase::enumType item_type)
 {
   signal_layout_item_added().emit(item_type);
 }
 
+Gtk::Widget* DataWidget::get_data_child_widget()
+{
+  return m_child;
+}
+
+const Gtk::Widget* DataWidget::get_data_child_widget() const
+{
+  return m_child;
+}
+ 
+ DataWidget::type_signal_open_details_requested DataWidget::signal_open_details_requested()
+ {
+   return m_signal_open_details_requested;
+ }
+ 
+ void DataWidget::on_button_open_details()
+ {
+   signal_open_details_requested().emit(get_value());
+ }
+ 
+ void DataWidget::on_button_select_id()
+ {
+   Gnome::Gda::Value chosen_id;
+   bool chosen = offer_related_record_id_find(chosen_id);
+   if(chosen)
+   {
+     set_value(chosen_id);
+     m_signal_edited.emit(chosen_id);
+   }
+ }
+
+ bool DataWidget::offer_related_record_id_find(Gnome::Gda::Value& chosen_id)
+ {
+  bool result = false;
+
+  //Initialize output variable:
+  chosen_id = Gnome::Gda::Value();
+
+  try
+  {
+    Glib::RefPtr<Gnome::Glade::Xml> refXml = Gnome::Glade::Xml::create(GLOM_GLADEDIR "glom.glade", "dialog_find_id");
+
+    Dialog_ChooseID* dialog = 0;
+    refXml->get_widget_derived("dialog_find_id", dialog);
+
+    if(dialog)
+    {
+      //dialog->set_document(get_document(), table_name, field);
+      dialog->set_transient_for(*get_application());
+      add_view(dialog);
+
+      //Discover the related table, in the relationship that uses this ID field:
+      Glib::ustring related_table_name;
+      sharedptr<LayoutItem_Field> layoutField = sharedptr<LayoutItem_Field>::cast_dynamic(get_layout_item());
+      if(layoutField)
+      {
+        sharedptr<Relationship> relationship = get_document()->get_field_used_in_relationship_to_one(m_table_name, layoutField->get_name());
+        if(relationship)
+          related_table_name = relationship->get_to_table();
+      }
+      else
+        g_warning("get_layout_item() was not a LayoutItem_Field");
+
+      dialog->init_db_details(related_table_name);
+
+
+      int response = dialog->run();
+      dialog->hide();
+      if(response == Gtk::RESPONSE_OK)
+      {
+        //Get the chosen field:
+        result = dialog->get_id_chosen(chosen_id);
+      }
+
+      remove_view(dialog);
+      delete dialog;
+    }
+  }
+  catch(const Gnome::Glade::XmlError& ex)
+  {
+    std::cerr << ex.what() << std::endl;
+  }
+
+  return result;
+}
