@@ -207,7 +207,7 @@ void DbAddDel::setup_menu()
     m_refContextEdit =  Gtk::Action::create("ContextEdit", Gtk::Stock::EDIT);
   else
     m_refContextEdit =  Gtk::Action::create("ContextEdit", m_open_button_title);
-    
+
   m_refActionGroup->add(m_refContextEdit,
     sigc::mem_fun(*this, &DbAddDel::on_MenuPopup_activate_Edit) );
 
@@ -303,7 +303,7 @@ bool DbAddDel::on_button_press_event_Popup(GdkEventButton *event)
     {
       //Double-click means edit.
       //Don't do this usually, because users sometimes double-click by accident when they just want to edit a cell.
-      
+
       //TODO: If the cell is not editable, handle the double-click as an edit/selection.
       //on_MenuPopup_activate_Edit();
       return false; //Not handled.
@@ -316,56 +316,7 @@ bool DbAddDel::on_button_press_event_Popup(GdkEventButton *event)
 Gtk::TreeModel::iterator DbAddDel::get_item_placeholder()
 {
   //Get the existing placeholder row, or add one if necessary:
-  Gtk::TreeModel::iterator iter = get_last_row();
-  if( get_is_placeholder_row(iter) )
-  {
-    return iter;
-  }
-  else
-  {
-    return add_item_placeholder();
-  }
-}
-
-Gtk::TreeModel::iterator DbAddDel::add_item_placeholder()
-{
-  Gtk::TreeModel::iterator iter;
-
-  //Placeholder rows are for adding new records.
-  if(!m_allow_add)
-    return iter;
-
-  iter = m_refListStore->append();
-  if(iter)
-  {
-    m_refListStore->set_is_placeholder(iter, true);
-    m_refListStore->set_key_value(iter, Gnome::Gda::Value()); //Remove temporary key value.
-  }
-
-  return iter;
-}
-
-Gtk::TreeModel::iterator DbAddDel::add_item(const Gnome::Gda::Value& valKey)
-{
-  if(!(get_model()))
-    return Gtk::TreeModel::iterator();
-
-  Gtk::TreeModel::iterator result = get_next_available_row_with_add_if_necessary();
-
-  if(result)
-  {
-    Gtk::TreeModel::Row treerow = *result;
-    if(treerow)
-    {
-      set_value_key(result, valKey);
-      m_refListStore->set_is_placeholder(result, false);
-      //treerow[*m_modelcolumn_placeholder] =  false; 
-    }
-  }
-
-  add_blank(); //if necessary
-
-  return result;
+  return m_refListStore->get_placeholder_row();
 }
 
 void DbAddDel::remove_all()
@@ -386,7 +337,7 @@ void DbAddDel::remove_all()
 }
 
 
-Gnome::Gda::Value DbAddDel::get_value(const Gtk::TreeModel::iterator& iter, const LayoutItem_Field& layout_item)
+Gnome::Gda::Value DbAddDel::get_value(const Gtk::TreeModel::iterator& iter, const sharedptr<const LayoutItem_Field>& layout_item)
 {
   Gnome::Gda::Value value;
 
@@ -399,7 +350,7 @@ Gnome::Gda::Value DbAddDel::get_value(const Gtk::TreeModel::iterator& iter, cons
       type_list_indexes list_indexes = get_column_index(layout_item);
       if(!list_indexes.empty())
       {
-        type_list_indexes::const_iterator iter = list_indexes.begin(); //Just get the first displayed instance of this field.
+        type_list_indexes::const_iterator iter = list_indexes.begin(); //Just get the first displayed instance of this field->
 
         const guint col_real = *iter + get_count_hidden_system_columns();
         treerow.get_value(col_real, value);
@@ -421,7 +372,7 @@ Gnome::Gda::Value DbAddDel::get_value_key_selected()
     return Gnome::Gda::Value();
 }
 
-Gnome::Gda::Value DbAddDel::get_value_selected(const LayoutItem_Field& layout_item)
+Gnome::Gda::Value DbAddDel::get_value_selected(const sharedptr<const LayoutItem_Field>& layout_item)
 {
   return get_value(get_item_selected(), layout_item);
 }
@@ -440,6 +391,9 @@ Gtk::TreeModel::iterator DbAddDel::get_item_selected()
 
 Gtk::TreeModel::iterator DbAddDel::get_row(const Gnome::Gda::Value& key)
 {
+  if(!m_refListStore)
+    return Gtk::TreeModel::iterator();
+
   for(Gtk::TreeModel::iterator iter = m_refListStore->children().begin(); iter != m_refListStore->children().end(); ++iter)
   {
     //Gtk::TreeModel::Row row = *iter;
@@ -464,7 +418,7 @@ bool DbAddDel::select_item(const Gtk::TreeModel::iterator& iter, guint column, b
 {
   if(!m_refListStore)
     return false;
-   
+
   InnerIgnore innerIgnore(this); //see comments for InnerIgnore class
 
   bool bResult = false;
@@ -475,7 +429,7 @@ bool DbAddDel::select_item(const Gtk::TreeModel::iterator& iter, guint column, b
     if(refTreeSelection)
     {
       refTreeSelection->select(iter);
-          
+
       Gtk::TreeModel::Path path = m_refListStore->get_path(iter);
 
       guint view_column_index = 0;
@@ -515,35 +469,6 @@ guint DbAddDel::get_count() const
   return iCount;
 }
 
-void DbAddDel::add_blank()
-{
-  bool bPreventUserSignals = get_prevent_user_signals();
-  set_prevent_user_signals(true);
-
-  bool bAddNewBlank = false;
-
-  if(get_allow_user_actions()) //The extra blank line is only used if the user may add items:
-  {
-    Gtk::TreeModel::iterator iter = get_last_row();
-    if(get_is_placeholder_row(iter))
-    {
-      bAddNewBlank  = false; //One already exists.
-    }
-    else
-    {
-      bAddNewBlank = true; // The last line isn't a placeholder. Add one.
-    }
-  }
-
-  if(bAddNewBlank)
-  {
-    add_item_placeholder();
-  }
-
-  set_prevent_user_signals(bPreventUserSignals);
-}
-
-
 guint DbAddDel::get_columns_count() const
 {
   return m_TreeView.get_columns().size();
@@ -604,7 +529,7 @@ void DbAddDel::construct_specified_columns()
 
       record.add( *pModelColumn );
 
-      fields.push_back(sharedptr<LayoutItem_Field>(new LayoutItem_Field(iter->m_field)));
+      fields.push_back( glom_sharedptr_clone(iter->m_field) );
 
       i++;
     }
@@ -616,10 +541,14 @@ void DbAddDel::construct_specified_columns()
   for(type_model_store::type_vec_fields::const_iterator iter = fields.begin(); iter != fields.end(); ++iter)
   {
     sharedptr<LayoutItem_Field> layout_item = *iter;
-    if( !(layout_item->get_has_relationship_name()) && (layout_item->m_field.get_primary_key()) )
+    if( !(layout_item->get_has_relationship_name()) )
     {
-      key_found = true;
-      break;
+      sharedptr<const Field> field_full = layout_item->get_full_field_details();
+      if(field_full && field_full->get_primary_key() )
+      {
+        key_found = true;
+        break;
+      }
     }
 
     ++column_index_key;
@@ -650,30 +579,30 @@ void DbAddDel::construct_specified_columns()
   //Add new View Colums:
   int model_column_index = 0; //Not including the hidden internal columns.
   int view_column_index = 0;
-  
+
   {
     GlomCellRenderer_Button* pCellButton = Gtk::manage(new GlomCellRenderer_Button());
     pCellButton->signal_clicked().connect(sigc::mem_fun(*this, &DbAddDel::on_cell_button_clicked));
-    
+
     const int viewcols_count = m_TreeView.append_column(Glib::ustring(), *pCellButton);
     m_treeviewcolumn_button = m_TreeView.get_column(viewcols_count - 1);
-    
+
     m_treeviewcolumn_button->property_visible() = get_allow_view_details();
-    
+
     ++view_column_index;
   }
-  
+
   for(type_vecModelColumns::iterator iter = vecModelColumns.begin(); iter != vecModelColumns.end(); ++iter)
   {
     DbAddDelColumnInfo& column_info = m_ColumnTypes[model_column_index];
-    if(column_info.m_visible && !(column_info.m_field.get_hidden())) //TODO: We shouldn't need both of these.
+    if(column_info.m_visible && !(column_info.m_field->get_hidden())) //TODO: We shouldn't need both of these.
     {
-      const Glib::ustring column_name = column_info.m_field.get_title_or_name();
-      const Glib::ustring column_id = column_info.m_field.get_name();
+      const Glib::ustring column_name = column_info.m_field->get_title_or_name();
+      const Glib::ustring column_id = column_info.m_field->get_name();
 
       int cols_count = 0;
       Gtk::CellRenderer* pCellRenderer = 0;
-      switch(column_info.m_field.m_field.get_glom_type())
+      switch(column_info.m_field->get_glom_type())
       {
         case(Field::TYPE_BOOLEAN):
         {
@@ -683,10 +612,10 @@ void DbAddDel::construct_specified_columns()
         }
         default:
         {
-          if(column_info.m_field.get_formatting_used().get_has_choices())
+          if(column_info.m_field->get_formatting_used().get_has_choices())
           {
             CellRendererList* rendererList = Gtk::manage( new CellRendererList() );
-            rendererList->set_restrict_values_to_list(column_info.m_field.get_formatting_used().get_choices_restricted());
+            rendererList->set_restrict_values_to_list(column_info.m_field->get_formatting_used().get_choices_restricted());
 
             pCellRenderer = rendererList;
           }
@@ -710,8 +639,8 @@ void DbAddDel::construct_specified_columns()
           //Make it editable:
           pCellRenderer->property_editable() = true;
 
-          if( column_info.m_field.m_field.get_glom_type() == Field::TYPE_NUMERIC )
-            pCellRenderer->property_xalign() = 1.0; //Align right.
+          if( column_info.m_field->get_glom_type() == Field::TYPE_NUMERIC )
+            pCellRenderer->property_xalign() = 1.0f; //Align right.
 
           //Connect to its signal:
           pCellRenderer->signal_edited().connect(
@@ -722,53 +651,53 @@ void DbAddDel::construct_specified_columns()
           {
             pCellRendererCombo->remove_all_list_items();
 
-            if(column_info.m_field.get_formatting_used().get_has_custom_choices())
+            if(column_info.m_field->get_formatting_used().get_has_custom_choices())
             {
               pCellRendererCombo->set_use_second(false); //Custom choices have only one column.
 
               //set_choices() needs this, for the numeric layout:
               //pCellRendererCombo->set_layout_item(get_layout_item()->clone(), table_name); //TODO_Performance: We only need this for the numerical format.
-              const FieldFormatting::type_list_values list_values = column_info.m_field.get_formatting_used().get_choices_custom();
+              const FieldFormatting::type_list_values list_values = column_info.m_field->get_formatting_used().get_choices_custom();
               for(FieldFormatting::type_list_values::const_iterator iter = list_values.begin(); iter != list_values.end(); ++iter)
               {
-                pCellRendererCombo->append_list_item( GlomConversions::get_text_for_gda_value(column_info.m_field.m_field.get_glom_type(), *iter, column_info.m_field.get_formatting_used().m_numeric_format) );
+                pCellRendererCombo->append_list_item( GlomConversions::get_text_for_gda_value(column_info.m_field->get_glom_type(), *iter, column_info.m_field->get_formatting_used().m_numeric_format) );
               }
             }
-            else if(column_info.m_field.get_formatting_used().get_has_related_choices())
+            else if(column_info.m_field->get_formatting_used().get_has_related_choices())
             {
-              Glib::ustring choice_relationship_name, choice_field, choice_second;
-              column_info.m_field.get_formatting_used().get_choices(choice_relationship_name, choice_field, choice_second);
-              if(!choice_relationship_name.empty() && !choice_field.empty())
+              sharedptr<Relationship> choice_relationship;
+              Glib::ustring choice_field, choice_second;
+              column_info.m_field->get_formatting_used().get_choices(choice_relationship, choice_field, choice_second);
+
+              if(choice_relationship && !choice_field.empty())
               {
-                const Relationship relationship = column_info.m_field.get_formatting_used().m_choices_related_relationship;
-                const Glib::ustring to_table = relationship.get_to_table();
+                const Glib::ustring to_table = choice_relationship->get_to_table();
 
                 const bool use_second = !choice_second.empty();
                 pCellRendererCombo->set_use_second(use_second);
 
-                LayoutItem_Field layout_field_second;
+                const sharedptr<LayoutItem_Field> layout_field_second = sharedptr<LayoutItem_Field>::create();
                 if(use_second)
                 {
                   Document_Glom* document = get_document();
                   if(document)
                   {
-                    Field field_second; //TODO: Actually show this in the combo:
-                    document->get_field(to_table, choice_second, field_second);
+                    sharedptr<Field> field_second = document->get_field(to_table, choice_second); //TODO: Actually show this in the combo:
 
-                    layout_field_second.m_field = field_second;
+                    layout_field_second->set_full_field_details(field_second);
 
-                    //We use the default formatting for this field.
+                    //We use the default formatting for this field->
                   }
                 }
 
                 GlomUtils::type_list_values_with_second list_values = GlomUtils::get_choice_values(column_info.m_field);
                 for(GlomUtils::type_list_values_with_second::const_iterator iter = list_values.begin(); iter != list_values.end(); ++iter)
                 {
-                  const Glib::ustring first = GlomConversions::get_text_for_gda_value(column_info.m_field.m_field.get_glom_type(), iter->first, column_info.m_field.get_formatting_used().m_numeric_format);
+                  const Glib::ustring first = GlomConversions::get_text_for_gda_value(column_info.m_field->get_glom_type(), iter->first, column_info.m_field->get_formatting_used().m_numeric_format);
 
                   Glib::ustring second;
                   if(use_second)
-                    second = GlomConversions::get_text_for_gda_value(layout_field_second.m_field.get_glom_type(), iter->second, layout_field_second.get_formatting_used().m_numeric_format);
+                    second = GlomConversions::get_text_for_gda_value(layout_field_second->get_glom_type(), iter->second, layout_field_second->get_formatting_used().m_numeric_format);
 
                   pCellRendererCombo->append_list_item(first, second);
                 }
@@ -808,17 +737,19 @@ void DbAddDel::construct_specified_columns()
   m_TreeView.columns_autosize();
 
   //Make sure there's a blank row after the database rows that have just been added.
-  add_blank();
+  //add_blank();
 }
 
 bool DbAddDel::refresh_from_database()
 {
   if(m_refListStore)
   {
-    Glib::RefPtr<Gtk::TreeModel> refNull;
+    //Glib::RefPtr<Gtk::TreeModel> refNull;
     bool result = m_refListStore->refresh_from_database(m_where_clause);
-    m_TreeView.set_model(refNull); //TODO: This causes a g_waring(): gtk_tree_view_unref_tree_helper: assertion `node != NULL' failed
-    //gtk_tree_view_set_model(m_TreeView.gobj(), 0); //This gives the same warning.
+    //m_TreeView.set_model(refNull); //TODO: This causes a g_warning(): gtk_tree_view_unref_tree_helper: assertion `node != NULL' failed
+    if(m_TreeView.get_model())
+      gtk_tree_view_set_model(m_TreeView.gobj(), 0); //This gives the same warning.
+
     m_TreeView.set_model(m_refListStore);
     return result;
   }
@@ -826,7 +757,17 @@ bool DbAddDel::refresh_from_database()
     return false;
 }
 
-void DbAddDel::set_value(const Gtk::TreeModel::iterator& iter, const LayoutItem_Field& layout_item, const Gnome::Gda::Value& value)
+bool DbAddDel::refresh_from_database_blank()
+{
+  if(m_refListStore)
+  {
+    m_refListStore->clear(); //Remove all rows.
+  }
+
+  return true;
+}
+
+void DbAddDel::set_value(const Gtk::TreeModel::iterator& iter, const sharedptr<const LayoutItem_Field>& layout_item, const Gnome::Gda::Value& value)
 {
   //g_warning("DbAddDel::set_value begin");
 
@@ -855,13 +796,13 @@ void DbAddDel::set_value(const Gtk::TreeModel::iterator& iter, const LayoutItem_
     }
 
     //Add extra blank if necessary:
-    add_blank();
+    //add_blank();
   }
-  
+
   //g_warning("DbAddDel::set_value end");
 }
 
-void DbAddDel::set_value_selected(const LayoutItem_Field& layout_item, const Gnome::Gda::Value& value)
+void DbAddDel::set_value_selected(const sharedptr<const LayoutItem_Field>& layout_item, const Gnome::Gda::Value& value)
 {
   set_value(get_item_selected(), layout_item, value);
 }
@@ -878,7 +819,7 @@ void DbAddDel::set_table_name(const Glib::ustring& table_name)
   m_table_name = table_name;
 }
 
-guint DbAddDel::add_column(const LayoutItem_Field& field)
+guint DbAddDel::add_column(const sharedptr<const LayoutItem_Field>& field)
 {
   InnerIgnore innerIgnore(this); //Stop on_treeview_columns_changed() from doing anything when it is called just because we add a new column.
 
@@ -888,10 +829,11 @@ guint DbAddDel::add_column(const LayoutItem_Field& field)
   //column_info.m_visible = visible;
 
   //Make it non-editable if it is auto-generated:
-  if(field.m_field.get_auto_increment())
+  sharedptr<const Field> field_full = field->get_full_field_details();
+  if(field_full && field_full->get_auto_increment())
     column_info.m_editable = false;
   else
-    column_info.m_editable = field.get_editable_and_allowed();
+    column_info.m_editable = field->get_editable_and_allowed();
 
   m_ColumnTypes.push_back(column_info);
 
@@ -910,25 +852,30 @@ void DbAddDel::set_where_clause(const Glib::ustring& where_clause)
   m_where_clause = where_clause;
 }
 
+Glib::ustring DbAddDel::get_where_clause() const
+{
+  return m_where_clause;
+}
+
 void DbAddDel::set_columns_ready()
 {
   m_columns_ready = true;
   construct_specified_columns();
 }
 
-DbAddDel::type_list_indexes DbAddDel::get_column_index(const LayoutItem_Field& layout_item) const
+DbAddDel::type_list_indexes DbAddDel::get_column_index(const sharedptr<const LayoutItem_Field>& layout_item) const
 {
   //TODO_Performance: Replace all this looping by a cache/map:
 
   type_list_indexes list_indexes;
 
-  const Glib::ustring field_name = layout_item.get_name();
-  const Glib::ustring relationship_name = layout_item.get_relationship_name();
+  const Glib::ustring field_name = layout_item->get_name();
+  const Glib::ustring relationship_name = layout_item->get_relationship_name();
 
   guint i = 0;
   for(type_ColumnTypes::const_iterator iter = m_ColumnTypes.begin(); iter != m_ColumnTypes.end(); ++iter)
   {
-    if( (iter->m_field.get_name() == field_name) && (iter->m_field.get_relationship_name() == relationship_name) )
+    if( (iter->m_field->get_name() == field_name) && (iter->m_field->get_relationship_name() == relationship_name) )
     {
       list_indexes.push_back(i);
     }
@@ -939,12 +886,12 @@ DbAddDel::type_list_indexes DbAddDel::get_column_index(const LayoutItem_Field& l
   return list_indexes;
 }
 
-LayoutItem_Field DbAddDel::get_column_field(guint column_index) const
+sharedptr<const LayoutItem_Field> DbAddDel::get_column_field(guint column_index) const
 {
   if(column_index < m_ColumnTypes.size())
     return m_ColumnTypes[column_index].m_field;
 
-  return LayoutItem_Field();
+  return sharedptr<const LayoutItem_Field>();
 }
 
 bool DbAddDel::get_prevent_user_signals() const
@@ -996,8 +943,7 @@ void DbAddDel::set_allow_delete(bool val)
 {
   m_allow_delete = val;
 }
-  
-  
+
 void DbAddDel::set_allow_user_actions(bool bVal)
 {
   m_bAllowUserActions = bVal;
@@ -1135,7 +1081,7 @@ void DbAddDel::on_treeview_cell_edited_bool(const Glib::ustring& path_string, in
     row.set_value(tree_model_column_index, value_new);
 
     //TODO: Did it really change?
-    
+
     //Is this an add or a change?:
 
     bool bIsAdd = false;
@@ -1223,22 +1169,21 @@ void DbAddDel::on_treeview_cell_edited(const Glib::ustring& path_string, const G
         set_prevent_user_signals(true); //Stops extra signal_user_changed.
 
         //Mark this row as no longer a placeholder, because it has data now. The client code must set an actual key for this in the signal_user_added() or m_signal_user_changed signal handlers.
-        //m_refListStore->set_is_placeholder(iter, false);
+        m_refListStore->set_is_not_placeholder(iter);
         //Don't mark this as not a placeholder, because it's still a placeholder until it has a key value.
 
-        add_item_placeholder(); //Add the next blank for the next user add, if necessary.
         set_prevent_user_signals(bPreventUserSignals);
 
         bIsAdd = true; //Signal that a new key was added.
       }
     }
 
-    const Field::glom_field_type field_type = m_ColumnTypes[model_column_index].m_field.m_field.get_glom_type();
+    const Field::glom_field_type field_type = m_ColumnTypes[model_column_index].m_field->get_glom_type();
     if(field_type != Field::TYPE_INVALID) //If a field type was specified for this column.
     {
       //Make sure that the entered data is suitable for this field type:
       bool success = false;
-      Gnome::Gda::Value value = GlomConversions::parse_value(field_type, new_text, m_ColumnTypes[model_column_index].m_field.get_formatting_used().m_numeric_format, success);
+      Gnome::Gda::Value value = GlomConversions::parse_value(field_type, new_text, m_ColumnTypes[model_column_index].m_field->get_formatting_used().m_numeric_format, success);
       if(!success)
       {
           //Tell the user and offer to revert or try again:
@@ -1363,20 +1308,6 @@ DbAddDel::type_signal_user_reordered_columns DbAddDel::signal_user_reordered_col
   return m_signal_user_reordered_columns;
 }
 
-Glib::ustring DbAddDel::string_escape_underscores(const Glib::ustring& text)
-{
-  Glib::ustring result;
-  for(Glib::ustring::const_iterator iter = text.begin(); iter != text.end(); ++iter)
-  {
-    if(*iter == '_')
-      result += "__";
-    else
-      result += *iter;
-  }
- 
-  return result;
-}
-
 void DbAddDel::on_treeview_button_press_event(GdkEventButton* event)
 {
   if(event->type == GDK_BUTTON_PRESS) //Whatever would cause cellrenderer activation.
@@ -1455,11 +1386,11 @@ guint DbAddDel::treeview_append_column(const Glib::ustring& title, Gtk::CellRend
   pViewColumn->set_min_width(100);
 
   //Save the extra ID, using the title if the column_id is empty:
-  Glib::ustring column_id = m_ColumnTypes[model_column_index].m_field.get_name();
+  Glib::ustring column_id = m_ColumnTypes[model_column_index].m_field->get_name();
   pViewColumn->set_column_id( (column_id.empty() ? title : column_id) );
 
   //TODO pViewColumn->signal_button_press_event().connect( sigc::mem_fun(*this, &DbAddDel::on_treeview_columnheader_button_press_event) );
-  
+
   return cols_count;
 }
 
@@ -1528,49 +1459,6 @@ bool DbAddDel::get_is_last_row(const Gtk::TreeModel::iterator& iter) const
     return false;
 }
 
-Gtk::TreeModel::iterator DbAddDel::get_next_available_row_with_add_if_necessary()
-{
-  Gtk::TreeModel::iterator result;
-
-  if(!m_refListStore)
-    return result;
-
-  bool bPreventUserSignals = get_prevent_user_signals();
-  set_prevent_user_signals(true);
-
-  if(get_allow_user_actions()) //The extra blank line is only used if the user may add items:
-  {
-    Gtk::TreeModel::iterator iter = get_last_row();
-
-    if(iter != get_model()->children().end())
-    {
-      //Look at the last row:
-      if( get_is_placeholder_row(iter))
-      {
-        result = iter;
-      }
-      else
-      {
-        // The last line isn't blank, so we can not use it. Add another one.
-        result = m_refListStore->append();
-      }
-    }
-    else
-    {
-       // This is the first line.
-       result = m_refListStore->append();
-    }
-  }
-  else
-  {
-     result = m_refListStore->append(); //Add a new blank line. There are no blank lines.
-  }
-
-  set_prevent_user_signals(bPreventUserSignals);
-
-  return result;
-}
-
 Gtk::TreeModel::iterator DbAddDel::get_last_row() const
 {
   return m_refListStore->get_last_row();
@@ -1596,7 +1484,7 @@ void DbAddDel::set_value_key(const Gtk::TreeModel::iterator& iter, const Gnome::
     if(!(GlomConversions::value_is_empty(value)))
     {
       //This is not a placeholder anymore, if it every was:
-      m_refListStore->set_is_placeholder(iter, false);
+      m_refListStore->set_is_not_placeholder(iter);
       //row[*m_modelcolumn_placeholder] = false;
     }
 
@@ -1662,12 +1550,12 @@ void DbAddDel::set_rules_hint(bool val)
   m_TreeView.set_rules_hint(val);
 }
 
-Field DbAddDel::get_key_field() const
+sharedptr<Field> DbAddDel::get_key_field() const
 {
   return m_key_field;
 }
 
-void DbAddDel::set_key_field(const Field& field)
+void DbAddDel::set_key_field(const sharedptr<Field>& field)
 {
   m_key_field = field;
 }
@@ -1682,7 +1570,7 @@ void DbAddDel::treeviewcolumn_on_cell_data(Gtk::CellRenderer* renderer, const Gt
     treerow->get_value(col_real, value);
 
     DbAddDelColumnInfo& column_info = m_ColumnTypes[model_column_index];
-    switch(column_info.m_field.m_field.get_glom_type())
+    switch(column_info.m_field->get_glom_type())
     {
       case(Field::TYPE_BOOLEAN):
       {
@@ -1696,7 +1584,7 @@ void DbAddDel::treeviewcolumn_on_cell_data(Gtk::CellRenderer* renderer, const Gt
         //TODO: Maybe we should have custom cellrenderers for time, date, and numbers.
         Gtk::CellRendererText* pDerived = dynamic_cast<Gtk::CellRendererText*>(renderer);
 
-        const Glib::ustring text = GlomConversions::get_text_for_gda_value(column_info.m_field.m_field.get_glom_type(), value, column_info.m_field.get_formatting_used().m_numeric_format);
+        const Glib::ustring text = GlomConversions::get_text_for_gda_value(column_info.m_field->get_glom_type(), value, column_info.m_field->get_formatting_used().m_numeric_format);
         //g_assert(text != "NULL");
         pDerived->property_text() = text;
 
@@ -1742,7 +1630,7 @@ void DbAddDel::on_cell_button_clicked(const Gtk::TreeModel::Path& path)
     const guint first_visible = get_count_hidden_system_columns();
     select_item(iter, first_visible, false /* start_editing */);
   }
-  
+
   on_MenuPopup_activate_Edit();
 }
 
